package at.bestsolution.rsd.rt.opentracing.zipkin;

import java.util.Map;

import org.eclipse.microprofile.config.ConfigProvider;
import org.osgi.service.component.annotations.Component;

import at.bestsolution.rsd.rt.opentracing.TraceProvider;
import brave.Tracing;
import brave.opentracing.BraveTracer;
import io.opentracing.Tracer;
import zipkin2.Span;
import zipkin2.reporter.AsyncReporter;
import zipkin2.reporter.okhttp3.OkHttpSender;

@Component
public class ZipkinTracingProvider implements TraceProvider {
	private volatile Tracer tracer;
	
	@Override
	public String getId() {
		return "zipkin";
	}

	private String getComponentName() {
		return ConfigProvider.getConfig().getOptionalValue("rsd.opentracing.component.name", String.class)
				.orElse("RSD Trace");
	}

	private String getHost() {
		return ConfigProvider.getConfig().getOptionalValue("rsd.opentracing.zipkin.host", String.class)
				.orElse("localhost");
	}

	private Integer getPort() {
		return ConfigProvider.getConfig().getOptionalValue("rsd.opentracing.zipkin.port", Integer.class).orElse(9411);
	}

	@Override
	public synchronized Tracer getTracer() {
		// XXX Detect configuration changes
		Tracer localTracer = tracer;
		if (localTracer == null) {
			synchronized (this) {
				localTracer = tracer;
				if (localTracer == null) {
					OkHttpSender sender = OkHttpSender.create("http://" + getHost() + ":" + getPort() + "/api/v2/spans");
					AsyncReporter<Span> spanReporter = AsyncReporter.create(sender);
					Tracing braveTracing = Tracing.newBuilder()
						.localServiceName(getComponentName())
						.spanReporter(spanReporter)
						.build();
					
					localTracer = tracer = BraveTracer.create(braveTracing);
				}
			}
		}

		return localTracer;
	}

	@Override
	public Map<String, String> spanContextMap(String jsonString){
		Map<String, String> map = TraceProvider.super.spanContextMap(jsonString);
		//uber-trace-id == jaeger key, so needs to be changed to zipkin
		if(map.containsKey("uber-trace-id")) {
			String jsonZipkinString = map.remove("uber-trace-id");
			String[] parts = jsonZipkinString.split(":");
			map.put("traceparent", parts[2] + "-" +parts[0] + "-" +parts[1] + "-" +parts[3]);
		}
		return map;
	}
}
